@@ -87,11 +87,7 @@ impl Decimal {
     ///
     /// `int_val` has at most 38 tens digits, `scale` ranges from `[-126, 130]`.
     #[inline]
-    pub const fn from_parts(
-        int_val: u128,
-        scale: i16,
-        negative: bool,
-    ) -> Result<Decimal, DecimalConvertError> {
+    pub const fn from_parts(int_val: u128, scale: i16, negative: bool) -> Result<Decimal, DecimalConvertError> {
         if int_val > MAX_I128_REPR as u128 {
             return Err(DecimalConvertError::Overflow);
         }
@@ -222,10 +218,7 @@ impl Decimal {
 
     /// Encodes `self` to `writer` as binary bytes.
     /// Returns total size on success, which is not larger than [`MAX_BINARY_SIZE`].
-    fn internal_encode<W: Write, const COMPACT: bool>(
-        &self,
-        mut writer: W,
-    ) -> std::io::Result<usize> {
+    fn internal_encode<W: Write, const COMPACT: bool>(&self, mut writer: W) -> std::io::Result<usize> {
         let int_bytes: [u8; 16] = self.int_val.to_le_bytes();
 
         let mut id = 15;
@@ -474,9 +467,7 @@ impl Decimal {
             let shift_scale = (digits - MAX_PRECISION) as i16;
             let dividend = int_val + ROUNDINGS[shift_scale as usize].low();
             let result = dividend / POWERS_10[shift_scale as usize].low();
-            return Some(unsafe {
-                Decimal::from_parts_unchecked(result.low(), scale - shift_scale, negative)
-            });
+            return Some(unsafe { Decimal::from_parts_unchecked(result.low(), scale - shift_scale, negative) });
         }
 
         Some(unsafe { Decimal::from_parts_unchecked(int_val.low(), scale, negative) })
@@ -489,7 +480,15 @@ impl Decimal {
         let e = other.scale - self.scale;
         debug_assert!(e > 0);
         if e as u32 > MAX_PRECISION {
-            return Some(*self);
+            if (e as usize) < POWERS_10.len() {
+                if let Some(self_int_val) = POWERS_10[e as usize].checked_mul(self.int_val) {
+                    if let Some(int_val) = self_int_val.checked_add(other.int_val) {
+                        return Decimal::adjust_scale(int_val, other.scale, negative);
+                    }
+                }
+            }
+
+            return Some(unsafe { Decimal::from_parts_unchecked(self.int_val, self.scale, negative) });
         }
 
         let self_int_val = U256::mul128(self.int_val, POWERS_10[e as usize].low());
@@ -509,9 +508,7 @@ impl Decimal {
 
         let int_val = U256::add128(self.int_val, other.int_val);
         if !int_val.is_decimal_overflowed() && self.scale >= 0 {
-            return Some(unsafe {
-                Decimal::from_parts_unchecked(int_val.low(), self.scale, negative)
-            });
+            return Some(unsafe { Decimal::from_parts_unchecked(int_val.low(), self.scale, negative) });
         }
 
         Decimal::adjust_scale(int_val, self.scale, negative)
@@ -524,7 +521,15 @@ impl Decimal {
         let e = other.scale - self.scale;
         debug_assert!(e > 0);
         if e as u32 > MAX_PRECISION {
-            return Some(*self);
+            if (e as usize) < POWERS_10.len() {
+                if let Some(self_int_val) = POWERS_10[e as usize].checked_mul(self.int_val) {
+                    if let Some(int_val) = self_int_val.checked_sub(other.int_val) {
+                        return Decimal::adjust_scale(int_val, other.scale, negative);
+                    }
+                }
+            }
+
+            return Some(unsafe { Decimal::from_parts_unchecked(self.int_val, self.scale, negative) });
         }
 
         let self_int_val = U256::mul128(self.int_val, POWERS_10[e as usize].low());
@@ -546,9 +551,7 @@ impl Decimal {
         }
 
         if self.int_val == 0 {
-            return Some(unsafe {
-                Decimal::from_parts_unchecked(other.int_val, other.scale, !negative)
-            });
+            return Some(unsafe { Decimal::from_parts_unchecked(other.int_val, other.scale, !negative) });
         }
 
         if self.scale != other.scale {
@@ -633,15 +636,9 @@ impl Decimal {
 
         let (self_int_val, shift_precision) = if other_precision > self_precision {
             let p = MAX_PRECISION + (other_precision - self_precision) as u32;
-            (
-                POWERS_10[p as usize] * self.int_val,
-                other_precision - self_precision,
-            )
+            (POWERS_10[p as usize] * self.int_val, other_precision - self_precision)
         } else {
-            (
-                U256::mul128(self.int_val, POWERS_10[MAX_PRECISION as usize].low()),
-                0,
-            )
+            (U256::mul128(self.int_val, POWERS_10[MAX_PRECISION as usize].low()), 0)
         };
 
         let negative = self.negative ^ other.negative;
@@ -728,7 +725,7 @@ impl Decimal {
         let mut last = result;
 
         loop {
-            let val = self.checked_div(result)?;
+            let val = self.checked_div(result)?.normalize();
             result = result.checked_add(val)?;
             result = result.checked_mul(Decimal::ZERO_POINT_FIVE)?;
 
@@ -1146,10 +1143,7 @@ mod tests {
             (9_9999_9999_9999_9999_9999_9999_9999_9999_9999_u128, -2),
             (99_9999_9999_9999_9999_9999_9999_9999_9999_9990_u128, -1),
         );
-        assert_normalize(
-            (12300, MIN_SCALE + 1),
-            (12300000000000000000000000000000000000, -92),
-        );
+        assert_normalize((12300, MIN_SCALE + 1), (12300000000000000000000000000000000000, -92));
     }
 
     #[test]
@@ -1226,5 +1220,7 @@ mod tests {
         );
         assert_sqrt("1e100", "1e50");
         assert_sqrt("1.01e100", "1.0049875621120890270219264912759576187e50");
+        assert_sqrt("1e-100", "1e-50");
+        assert_sqrt("1.01e-100", "1.0049875621120890270219264912759576187e-50");
     }
 }
