@@ -17,9 +17,7 @@
 use crate::decimal::Decimal;
 use std::convert::TryFrom;
 use std::iter::{Product, Sum};
-use std::ops::{
-    Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign,
-};
+use std::ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign};
 
 impl Neg for Decimal {
     type Output = Decimal;
@@ -402,39 +400,32 @@ mod tests {
         assert_add("0.000000001", "100000000", "100000000.000000001");
         assert_add("123456789.987654321", "-123456789.987654321", "0");
         assert_add("987654321.123456789", "-987654321.123456789", "0");
-        assert_add(
-            "123456789.987654321",
-            "987654321.123456789",
-            "1111111111.11111111",
-        );
+        assert_add("123456789.987654321", "987654321.123456789", "1111111111.11111111");
         assert_add("123456789.987654321", "00000.00000", "123456789.987654321");
-        assert_add(
-            "123456789.987654321",
-            "-987654321.123456789",
-            "-864197531.135802468",
-        );
+        assert_add("123456789.987654321", "-987654321.123456789", "-864197531.135802468");
         assert_add("00000.00000", "987654321.123456789", "987654321.123456789");
         assert_add("00000.00000", "00000.00000", "0");
+        assert_add("00000.00000", "-987654321.123456789", "-987654321.123456789");
+        assert_add("-123456789.987654321", "987654321.123456789", "864197531.135802468");
+        assert_add("-123456789.987654321", "00000.00000", "-123456789.987654321");
+        assert_add("-123456789.987654321", "-987654321.123456789", "-1111111111.11111111");
+        assert_add("-1e28", "-1e122", "-1e122");
         assert_add(
-            "00000.00000",
-            "-987654321.123456789",
-            "-987654321.123456789",
+            "0",
+            "0.00038758507034516434217352824488052003540",
+            "0.00038758507034516434217352824488052003540",
         );
         assert_add(
-            "-123456789.987654321",
-            "987654321.123456789",
-            "864197531.135802468",
+            "1",
+            "0.00038758507034516434217352824488052003540",
+            "1.00038758507034516434217352824488052",
         );
         assert_add(
-            "-123456789.987654321",
-            "00000.00000",
-            "-123456789.987654321",
-        );
-        assert_add(
-            "-123456789.987654321",
-            "-987654321.123456789",
-            "-1111111111.11111111",
+            "99999999999999999999999999999999999999e100",
+            "99999999999999999999999999999999999999e-100",
+            "99999999999999999999999999999999999999e100",
         );
+        assert_add("1e-31", "1e-105", "1e-31");
     }
 
     #[test]
@@ -452,12 +443,7 @@ mod tests {
             assert_eq!(result2, expected2);
         }
 
-        assert_sub(
-            "0.000000001",
-            "100000000",
-            "-99999999.999999999",
-            "99999999.999999999",
-        );
+        assert_sub("0.000000001", "100000000", "-99999999.999999999", "99999999.999999999");
         assert_sub(
             "123456789.987654321",
             "123456789.987654321",
@@ -519,6 +505,25 @@ mod tests {
             "864197531.135802468",
             "-864197531.135802468",
         );
+        assert_sub("-1e28", "-1e122", "1e122", "-1e122");
+        assert_sub(
+            "0",
+            "0.00038758507034516434217352824488052003540",
+            "-0.00038758507034516434217352824488052003540",
+            "0.00038758507034516434217352824488052003540",
+        );
+        assert_sub(
+            "1",
+            "0.00038758507034516434217352824488052003540",
+            "0.99961241492965483565782647175511947996",
+            "-0.99961241492965483565782647175511947996",
+        );
+        assert_sub(
+            "99999999999999999999999999999999999999e100",
+            "99999999999999999999999999999999999999e-100",
+            "99999999999999999999999999999999999999e100",
+            "-99999999999999999999999999999999999999e100",
+        );
     }
 
     #[test]
@@ -653,42 +658,18 @@ mod tests {
         assert_rem("100000000", "0.000000001", "0.000000000");
         assert_rem("123456789.987654321", "123456789.987654321", "0");
         assert_rem("987654321.123456789", "987654321.123456789", "0");
-        assert_rem(
-            "123456789.987654321",
-            "987654321.123456789",
-            "123456789.987654321",
-        );
+        assert_rem("123456789.987654321", "987654321.123456789", "123456789.987654321");
         assert_rem("987654321.123456789", "123456789.987654321", "1.222222221");
         assert_rem("00000.00000", "123456789.987654321", "0");
-        assert_rem(
-            "123456789.987654321",
-            "-987654321.123456789",
-            "123456789.987654321",
-        );
-        assert_rem(
-            "-987654321.123456789",
-            "123456789.987654321",
-            "-1.222222221",
-        );
+        assert_rem("123456789.987654321", "-987654321.123456789", "123456789.987654321");
+        assert_rem("-987654321.123456789", "123456789.987654321", "-1.222222221");
         assert_rem("00000.00000", "987654321.123456789", "0.000000000");
         assert_rem("00000.00000", "-987654321.123456789", "0.000000000");
-        assert_rem(
-            "-123456789.987654321",
-            "987654321.123456789",
-            "-123456789.987654321",
-        );
+        assert_rem("-123456789.987654321", "987654321.123456789", "-123456789.987654321");
         assert_rem("987654321.123456789", "-123456789.987654321", "1.222222221");
         assert_rem("00000.00000", "-123456789.987654321", "0.000000000");
-        assert_rem(
-            "-123456789.987654321",
-            "-987654321.123456789",
-            "-123456789.987654321",
-        );
-        assert_rem(
-            "-987654321.123456789",
-            "-123456789.987654321",
-            "-1.222222221",
-        );
+        assert_rem("-123456789.987654321", "-987654321.123456789", "-123456789.987654321");
+        assert_rem("-987654321.123456789", "-123456789.987654321", "-1.222222221");
         assert_rem("100", "5", "0");
         assert_rem("2e1", "1", "0");
         assert_rem("2", "1", "0");
@@ -716,11 +697,7 @@ mod tests {
         assert_rem("5e-42", "3e-84", "2e-84");
         assert_rem("5e126", "3e-130", "2e88");
         assert_rem("4e126", "3e-130", "1e88");
-        assert_rem(
-            "99999999999999999999999999999999999999e126",
-            "7e-130",
-            "2e88",
-        );
+        assert_rem("99999999999999999999999999999999999999e126", "7e-130", "2e88");
         assert_rem("3", "5e42", "3");
     }
 
@@ -740,10 +717,7 @@ mod tests {
     #[test]
     fn test_product() {
         fn assert_product(vals: &[&str], expected: &str) {
-            let result: Decimal = vals
-                .iter()
-                .map(|val| val.parse::<Decimal>().unwrap())
-                .product();
+            let result: Decimal = vals.iter().map(|val| val.parse::<Decimal>().unwrap()).product();
             let expected = expected.parse::<Decimal>().unwrap();
             assert_eq!(result, expected);
         }
